$STORAGE:2
      PROGRAM SS2DGAMH
C -----------------------------------------------------------------------
C***  PROGRAM TO COMPUTE SEMIVARIOGRAMS (GAMMA(H))
C***  THIS PROGRAM WAS WRITTEN BY H.P. KNUDSEN OF THE UNIVERSITY OF
C***  ARIZONA.  ORIGINAL SOURCE CODE AND DOCUMENTATION ARE GIVEN IN
C***  Knudsen, H.P., and Kim, Y.C., 1977, A short course on geostatist-
C***      ical ore reserve estimation: College of mines, University of
C***      Arizona, Tucson, Arizona, 202 pages and appendices.
C***  THE PROGRAM WAS MODIFIED BY W. D. GRUNDY, U.S. GEOLOGICAL SUR-
C***  VEY, TO REDUCE RUNNING TIME ON LARGE DATA SETS AND TO ACCEPT
C***  DATA FROM AN UNFORMATTED (STATPAC) INPUT FILE.
C***  REVISED NOVEMBER 1985 TO INCORPORATE OPTIONAL ROBUST
C***  ESTIMATION OF THE VARIOGRAM(S).  DESCRIPTION OF ROBUST
C***  ESTIMATION IS GIVEN IN
C***  Krige, Daniel G., and Eduardo J. Magri, 1982, Studies of the
C***      effects of outliers and data transformations on variogram
C***      estimates for a base metal and a gold ore body: Mathematical
C***      Geology, Vol.14, No. 6, pp 557-564.
C***  THE ROBUST VARIOGRAM OPTION CAN ONLY BE SELECTED IF THE GAMMA(H)
C***  DISPLAY AND NOT THE MOMENT CENTER DISPLAY IS REQUESTED.
C***  THE ROBUST ESTIMATOR IS USED FOR DETECTING OUTLIER DATA VALUES
C***  USING PROGRAM SS2DOUTL.  THE ROBUST VARIOGRAM SHOULD NOT BE USED
C***  IN ANY OTHER PROGRAMS OF THE SS2D SERIES.
C***
C***  THE PROGRAM COMPUTES THE FOLLOWING FORMULAS AND WRITES THEM TO
C***  A CHARACTER OUTPUT FILE IN TABULAR AND GRAPHIC FORM:
C***
C***  GAMMA(H)=(CUM.DIFFERENCE SQUARED)/(2*(NO. OF SAMPLES))
C***
C***  AS AN ALTERNATIVE ESTIMATE OF GAMMA(H) (MOMENT CENTER) :
C***  GAMMA(H)=(CUM (DISTANCE*DIFF.SQUARED))/(2*(CUM. DISTANCE))
C***  THIS OPTION IS USEFUL WHEN SAMPLE LOCATIONS ARE TIGHTLY CLUSTERED
C***  AND SEPARATED INTO DISTINCT CLUMPS.  WITH GRIDDED DATA OR NEARLY
C***  GRIDDED DATA, THE MOMENT CENTER VALUES ARE ALMOST EXACTLY THE
C***  SAME AS THE ORDINARY VALUES OF GAMMA(H).
C***
C***  AS AN OPTIONAL COMPUTATION :
C***  RELATIVE GAMMA(H)=GAMMA(H)/(SAMPLE MEAN SQUARED)
C***
C***  AVERAGE DISTANCE=(CUM. DISTANCE)/(NO. OF SAMPLES)
C***
C***  "DRIFT"=(CUM.DIFFERENCE)/(NO. OF SAMPLES)
C***  NOTE: THE USER MUST MAKE AN INDIVIDUAL DECISION ON THE INTERPRE-
C***  TATION AND USEFULNESS OF THIS "STATISTIC".
C***
C***  SEE COMMENTS BELOW FOR FORTRAN VARIABLE NAMES ASSIGNED TO THE
C***  TERMS IN THE ABOVE FORMULAS.
C***  AS AN OPTION, THE PROGRAM WRITES OUT A CHARACTER DISK FILE OF
C***  THE ABOVE VALUES WHICH CAN BE USED FOR SCREEN PLOTTING.
C***
C***
C***  PROGRAM COMPUTES VARIOGRAMS IN ONE TO FIVE DIRECTIONS.  VALUES
C***  ARE COMPUTED FOR UP TO 20 DISTANCE CLASS INTERVALS.  THE USER
C***  SPECIFIES THE SIZE OF THESE INTERVALS IN A PROGRAM CONTROL FILE.
C***  THE USER ALSO SPECIFIES THE ALLOWABLE SPREAD IN DEGREES FROM
C***  THE DIRECTION IN WHICH THE VARIOGRAM IS COMPUTED FOR DATA TO
C***  BE INCLUDED IN THE COMPUTATIONS. TO COMPUTE THE AVERAGE VARIOGRAM
C***  IN ALL DIRECTIONS, AN ANGULAR TOLERANCE (WINDOW) OF 90 DEGREES
C***  MUST BE SPECIFIED.
C***
C***
C***                      REQUIRED INPUT FILES
C***
C***  THIS PROGRAM REQUIRES A PREPROCESSED STATPAC INPUT FILE WHICH
C***  CONTAINS CARTESIAN X- AND Y-COORDINATES TOGETHER WITH THE
C***  OBSERVED VALUES OF THE REGIONALIZED VARIABLE.  PREPROCESSING
C***  IS DONE BY PROGRAM SS2DPREP, WHICH SORTS AN INPUT STATPAC FILE
C***  IN ASCENDING ORDER OF X- AND Y-COORDINATES, AND IF NO DUPLICATED
C***  COORDINATE PAIRS ARE FOUND, WRITES THE SORTED DATA TO THE
C***  PREPROCESSED STATPAC FILE.
C***
C***  THIS PROGRAM ALSO REQUIRES AN ASCII CONTROL FILE ON DISK WHICH
C***  CONTAINS USER-SPECIFICATIONS FOR THE VARIOGRAM RUN.  THIS ASCII
C***  CONTROL FILE IS CREATED BY PROGRAM SS2DVCTL.  FOR DESCRIPTION OF
C***  THE CONTROL FILE, SEE COMMENTS IN SS2DVCTL.FOR.
C***
C***  NOTE: THIS PROGRAM REJECTS QUALIFIED DATA VALUES, SO ONLY
C***  UNQUALIFIED DATA ARE ACTUALLY PROCESSED.
C***   FILE NAME REQUIREMENTS
C***        THE STATPAC INPUT FILE MUST HAVE THE EXTENSION .UPR
C***        THE VARIOGRAM CONTROL FILE MUST HAVE THE EXTENSION .FVC
C***        THE CHARACTER OUTPUT FILE FOR PRINTER DISPLAY IS
C***        AUTOMATICALLY GIVEN THE EXTENSION .FGA (Formatted GAmh).
C***        THE OPTIONAL CHARACTER FILE FOR PLOTTING IS AUTOMATICALLY
C***        GIVEN THE EXTENSION .FGP (Formatted Gamh Plot).
C***
C***  MAJOR VARIABLES USED IN PROGRAM SS2DGAMH
C***     MAXN         MAXIMUM NO OF DATA POINTS ALLOWED.
C***                  IT IS CURRENTLY SET TO 1500.
C***     XE(MAXN)     ARRAY OF X-COORDINATES (EASTINGS)
C***     YN(MAXN)     ARRAY OF Y-COORDINATES (NORTHINGS)
C***     GD(MAXN)     ARRAY OF SAMPLE VALUES (ASSAY VALUES)
C***     IDATA(MAXN)  INDICATOR VARIABLE WHICH FLAGS THE SAMPLE POINTS
C***                  USED IN COMPUTING THE VARIOGRAM(S)
C***     XDIF         DIFFERENCE IN EAST COORDINATES BETWEEN TWO POINTS
C***     YDIF         DIFFERENCE IN NORTH COORDINATES   "     "    "
C***     Q            DIFFERENCE IN SAMPLE VALUES       "     "     "
C***     QSQR         Q SQUARED
C***     DIS          DISTANCE BETWEEN TWO SAMPLES
C***     DISQ         DIS TIMES Q SQUARED
C***                  NEXT NINE ARRAYS ARE DIMENSIONED (5,20)
C***     DISCUM(I,J)               CUMULATIVE DISTANCE FOR I-TH
C***                               DIRECTION, J-TH CLASS INTERVAL
C***     DIFCUM(I,J)               CUMULATIVE Q
C***     QSQCUM(I,J)               CUMULATIVE QSQR
C***     DDFCUM(I,J)               CUMULATIVE DISQ
C***     NUMCUM(I,J)               CUMULATIVE NUMBER OF SAMPLES
C***     AVGDIS(I,J)               AVERAGE DISTANCE BETWEEN SAMPLE
C***                               POINTS, I-TH DIRECTION, J-TH CLASS
C***                               INTERVAL.
C***     DRIFT (I,J)               "DRIFT" MEASURE
C***     GAMMAH(I,J)               GAMMA(H)
C***     CTRMNT(I,J)               MOMENT CENTER ESTIMATE OF GAMMA(H)
C***     GDBAR        MEAN VALUE OF SAMPLES
C***     VARI         VARIANCE OF SAMPLES
C***     IPT          DISK UNIT CONTAINING VARIOGRAM CONTROL FILE
C***     KI           DISK UNIT CONTAINING SAMPLE LOCATIONS AND ASSAYS
C***     IOUT         DISK UNIT FOR PRINTED OUTPUT OF VARIOGRAMS
C***     IPCH         DISK UNIT FOR GRAPHIC PLOTTING DATA
C***
C***  LINKS REQUIRED: ASCIIZ, BYTMOV, CHEKER, CHRKNT, CKSAFE, CLREOL,
C***                  CONCAT, CRSOFF, CRSON, DATE, DELAY, ERRCHK,
C***                  FNDCUR, FSTDSP, INT21H, INTR, IOFS, ISEG, MDYHMS,
C***                  ODOMTR, PCHCEN, SAVRST, SCROLL, SOUND, TICKER,
C***                  TIME, UNPK
C***  NOTE: FSTDSP WRITES DIRECTLY TO VIDEO RAM
C***
C -----------------------------------------------------------------------
      INTEGER*4 ITIME1,ITIME2
      LOGICAL FLAG,ROBUST
      CHARACTER*32 UNIT08,UNIT10,UNIT20,UNIT21
      CHARACTER*10 HMS
      CHARACTER*8 MDY,IDSTP
      CHARACTER*4 EXT,HEAD,WKDAY,IROB
      CHARACTER*1 IANS
      INTEGER*4 NUMCUM
      COMMON XE(1500),YN(1500),GD(1500),IDATA(1500),AVGDIS(5,20),
     1DRIFT(5,20),NUMCUM(5,20),GAMMAH(5,20),CTRMNT(5,20),
     2HEAD(20),LOG,CLAS,DLIM,ANG(5),GDBAR,VARI,HMS,MDY,WKDAY,
     3STD,N,IPC,LL,SPR(5),ISCAL,TMAX,NREL,IOUT,IPT,IROB,
     4IPCH,KI,IDSTP(2),MAXN,NDIR,ISEL,XMIN,XMAX,YMIN,YMAX
      DIMENSION DANG(5),DSPR(5),CST(5),SIT(5),CSPR(5),
     1DISCUM(5,20),DIFCUM(5,20),QSQCUM(5,20),DDFCUM(5,20)
      EQUIVALENCE(GAMMAH(1,1),QSQCUM(1,1)),(CTRMNT(1,1),DDFCUM(1,1)),
     1(AVGDIS(1,1),DISCUM(1,1))
      DATA EXT/'.FGP'/
C -----------------------------------------------------------------------
C***  START TIMING OF PROGRAM RUN AND INITIALIZE REQUIRED VARIABLES.
C -----------------------------------------------------------------------
      CALL DATE(WKDAY,MDY)
      CALL TICKER(ITIME1)
      CALL TIME(HMS)
      MAXN=1500
      IPT=20
      KI=14
      IOUT=21
      IPCH=8
         DO 10 I=1,MAXN
         IDATA(I)=0
   10    CONTINUE
         DO 30 J=1,20
            DO 20 I=1,5
            DISCUM(I,J)=0.
            DIFCUM(I,J)=0.
            QSQCUM(I,J)=0.
            DDFCUM(I,J)=0.
            NUMCUM(I,J)=0
   20       CONTINUE
   30    CONTINUE
C -----------------------------------------------------------------------
C***  OPEN INPUT AND OUTPUT FILES
C -----------------------------------------------------------------------
      CALL ATTACH(KI,IPT,IOUT,UNIT10,UNIT20,UNIT21)
C -----------------------------------------------------------------------
C***  READ IDENTIFIER FOR VARIOGRAM RUN FROM CONTROL FILE
C -----------------------------------------------------------------------
      READ(IPT,'(20A4)') HEAD
C -----------------------------------------------------------------------
C***  READ SELECTION OPTIONS FROM CONTROL FILE
C -----------------------------------------------------------------------
      READ(IPT,'(6(I1,1X),F12.3,1X,F15.5)') LOG,IPC,LL,ISCAL,NREL,
     1NDIR,CLAS,TMAX
      CLAS=CLAS+.0001*CLAS
      IF(IPC.EQ.0) GO TO 50
   40 WRITE(*,'('' '',''GIVE A NAME FOR THE OUTPUT PLOTTER FILE: ''
     1,\)')
      READ(*,'(A)') UNIT08
      CALL CHEKER(UNIT08,EXT,FLAG)
      IF(FLAG) GO TO 40
C -----------------------------------------------------------------------
C***  CHECK IF PLOTTER FILE EXISTS.  IF IT DOES, WRITE ERROR MESSAGE.
C -----------------------------------------------------------------------
      CALL CKSAFE(UNIT08,1,FLAG)
      IF(FLAG) GO TO 40
      OPEN(IPCH,FILE=UNIT08,STATUS='NEW',FORM='FORMATTED')
C -----------------------------------------------------------------------
C***  READ DIRECTION AND ANGULAR DEVIATION RECORD(S) FROM CONTROL FILE.
C***  THERE MAY BE AS MANY AS FIVE OF THESE RECORDS.
C -----------------------------------------------------------------------
   50    DO 60 I=1,NDIR
         READ(IPT,'(F6.2,5X,F6.2)') ANG(I),SPR(I)
         DANG(I)=ANG(I)*1.7453292D-02
         CST(I)=COS(DANG(I))
         SIT(I)=SIN(DANG(I))
         DSPR(I)=(SPR(I)+.001)*1.7453292D-02
         CSPR(I)=COS(DSPR(I))
   60    CONTINUE
      DLIM=20.*CLAS
      DLIM2=DLIM*DLIM
C -----------------------------------------------------------------------
C***  READ COORDINATE SELECTION OPTION FROM CONTROL FILE.
C -----------------------------------------------------------------------
      READ(IPT,'(I1,1X,4(F12.3,1X))') ISEL,YMIN,YMAX,XMIN,XMAX
C -----------------------------------------------------------------------
C***  FIND OUT IF ROBUST VARIOGRAM ESTIMATOR IS WANTED
C -----------------------------------------------------------------------
      ROBUST=.FALSE.
      IROB='NO'
         IF(LL.EQ.0) THEN
         WRITE(*,'('' '',''DO YOU WANT A ROBUST ESTIMATOR OF THE '',
     1   ''VARIOGRAMS? Y/N '',\)')
         READ(*,'(A1)') IANS
            IF(IANS.EQ.'Y'.OR.IANS.EQ.'y') THEN
            ROBUST=.TRUE.
            IROB='YES'
            ENDIF
         ENDIF
C -----------------------------------------------------------------------
C***  READ IN ASSAY DATA FROM STATPAC FILE.
C -----------------------------------------------------------------------
      CALL RDN
C -----------------------------------------------------------------------
C***  CHECK TO SEE IF STATPAC INPUT FILE WAS PREPROCESSED.  IF IT
C***  WAS NOT PREPROCESSED, STOP EXECUTION OF PROGRAM.
C -----------------------------------------------------------------------
         DO 70 I=2,N
         IF(YN(I).LT.YN(I-1)) STOP 'INPUT FILE WAS NOT PREPROCESSED'
   70    CONTINUE
C -----------------------------------------------------------------------
C***  ON OPTION, CONVERT ASSAY VALUES TO BASE 10 LOGARITHMS
C -----------------------------------------------------------------------
         IF (LOG.EQ.1) THEN
         WRITE(*,'('' '',''TRANSFORMING ASSAY VALUES TO LOGARITHMS'')')
            DO 80 I=1,N
            GD(I)=ALOG10(GD(I))
   80       CONTINUE
         ENDIF
C -----------------------------------------------------------------------
C***  BEGIN COMPUTATION OF VARIOGRAMS
C -----------------------------------------------------------------------
      WRITE(*,'('' '',''      ....COMPUTING VARIOGRAMS'')')
      CALL FNDCUR(IROW,ICOL,0)
      NM1=N-1
         DO 120 I=1,NM1
         Y1=YN(I)
         X1=XE(I)
         IP1=I+1
            DO 100 J=IP1,N
            Y2=YN(J)
            X2=XE(J)
            YDIF=Y1-Y2
            IF(ABS(YDIF).GT.DLIM) GO TO 110
            XDIF=X1-X2
            IF(ABS(XDIF).GT.DLIM) GO TO 100
            DIS2=YDIF*YDIF+XDIF*XDIF
            IF(DIS2.GT.DLIM2) GO TO 100
            DIS=SQRT(DIS2)
            IF(DIS.EQ.0.) GO TO 200
            Q=GD(I)-GD(J)
            IF(ROBUST) QSQR=SQRT(ABS(Q))
            IF(.NOT.ROBUST) QSQR=Q*Q
C -----------------------------------------------------------------------
C***  COMPARE DIRECTION WITH ACCEPTABLE DIRECTIONS
C -----------------------------------------------------------------------
               DO 90 L=1,NDIR
               DOT=(XDIF*CST(L)+YDIF*SIT(L))/DIS
               ADOT=ABS(DOT)
               IF(ADOT.LT.CSPR(L))GO TO 90
               K=DIS/CLAS+1
C -----------------------------------------------------------------------
C***  KEEP TRACK OF DATA POINTS USED IN COMPUTING VARIOGRAMS
C -----------------------------------------------------------------------
               IDATA(I)=1
               IDATA(J)=1
               Q=SIGN(Q,DOT)
               DISQ=DIS*Q*Q
               DISCUM(L,K)=DISCUM(L,K)+DIS
               DIFCUM(L,K)=DIFCUM(L,K)+Q
               QSQCUM(L,K)=QSQCUM(L,K)+QSQR
               DDFCUM(L,K)=DDFCUM(L,K)+DISQ
               NUMCUM(L,K)=NUMCUM(L,K)+1
   90          CONTINUE
  100       CONTINUE
  110    IF(MOD(I,10).EQ.0) CALL ODOMTR(IROW,I)
  120    CONTINUE
      WRITE(*,'(''+'',I5,)') N
      WRITE(*,'('' '',''COMPUTING MEAN AND STANDARD DEVIATION'')')
C -----------------------------------------------------------------------
C***  COMPUTE MEAN AND STANDARD DEVIATION OF SAMPLES.
C -----------------------------------------------------------------------
      SUM=0.0
      SUM2=0.0
      IN=0
         DO 130 I=1,N
         IF(IDATA(I).NE.1)GO TO 130
         IN=IN+1
         SUM=SUM+GD(I)
         SUM2=SUM2+GD(I)*GD(I)
  130    CONTINUE
      N=IN
      VARI=(FLOAT(N)*SUM2-SUM*SUM)/(FLOAT(N)*FLOAT(N-1))
      GDBAR=SUM/FLOAT(N)
      REL=1./(GDBAR*GDBAR)
      STD=SQRT(VARI)
         DO 160 I=1,20
            DO 150 L=1,NDIR
            AN=FLOAT(NUMCUM(L,I))
            IF(AN.EQ.0.)GO TO 150
            DRIFT(L,I)=DIFCUM(L,I)/AN
C -----------------------------------------------------------------------
C***  AS AN OPTION, COMPUTE RELATIVE VARIOGRMS
C -----------------------------------------------------------------------
            IF(NREL.EQ.0)GO TO 140
            QSQCUM(L,I)=QSQCUM(L,I)*REL
            DDFCUM(L,I)=DDFCUM(L,I)*REL
  140       IF(.NOT.ROBUST) GAMMAH(L,I)=QSQCUM(L,I)/(2.*AN)
               IF(ROBUST) THEN
               QSQCUM(L,I)=(QSQCUM(L,I)/AN)**4/
     1         (0.457+0.494/AN+0.045/(AN*AN))
               GAMMAH(L,I)=QSQCUM(L,I)*0.5
               ENDIF
            IF(DDFCUM(L,I).EQ.0.) GO TO 150
            CTRMNT(L,I)=DDFCUM(L,I)/(DISCUM(L,I)*2.0)
            AVGDIS(L,I)=DISCUM(L,I)/AN
  150       CONTINUE
  160    CONTINUE
C -----------------------------------------------------------------------
C***  WRITE OUT RESULTS OF VARIOGRAM TO DISK FILE
C -----------------------------------------------------------------------
      WRITE(*,'('' '',''WRITING RESULTS ON DISK FILE(S)'')')
      IF(LL.EQ.0) CALL VAROUT(GAMMAH)
      IF(LL.EQ.1) CALL VAROUT(CTRMNT)
      WRITE(*,170)
  170 FORMAT(' ',/)
      WRITE(*,'('' THE OUTPUT FILE(S) HAVE BEEN WRITTEN'',/,
     1'' THE STATPAC INPUT FILE WAS '',A32,/,'' THE VARIOGRAM '',
     2''CONTROL FILE WAS '',A32,/,'' THE VARIOGRAM OUTPUT FILE '',
     3''IS '',A32)') UNIT10,UNIT20,UNIT21
      IF(IPC.NE.0) WRITE(*,'('' THE CHARACTER PLOTTER FILE IS '',
     1A32)') UNIT08
         DO 180 I=1,5
         CALL SOUND(1000,25)
  180    CONTINUE
C -----------------------------------------------------------------------
C***  COMPUTE RUNNING TIME OF PROGRAM
C -----------------------------------------------------------------------
      CALL TICKER(ITIME2)
      RTIME=FLOAT(ITIME2-ITIME1)/6000.
      WRITE(*,190) RTIME
  190 FORMAT(' PROGRAM ACTIVE TIME WAS ',F6.2,' MINUTES')
      STOP 'NORMAL END OF PROGRAM'
  200 WRITE(*,210) I,J
  210 FORMAT(' DUPLICATED LOCATIONS WERE DETECTED AT STATPAC ROWS ',I4,'
     1AND ',I4,/,' THERE MAY BE OTHER DUPLICATES.  SEARCH FOR ALL DUPLI
     2CATES USING ',/,' PROGRAM SS2DPREP.  EXECUTION IS HALTED.')
      STOP
      END
      SUBROUTINE ATTACH(KI,IPT,IOUT,UNIT14,UNIT20,UNIT21)
C -----------------------------------------------------------------------
C***  THIS SUBROUTINE OPENS THE INPUT AND OUTPUT FILES FOR
C***  PROGRAM SS2DGAMH
C -----------------------------------------------------------------------
      LOGICAL FLAG
      CHARACTER*4 EXT0,EXT1,EXT2
      CHARACTER*32 UNIT14,UNIT20,UNIT21
      DATA EXT0/'.UPR'/,EXT1/'.FVC'/,EXT2/'.FGA'/
   10 WRITE(*,'('' '',''WHAT IS THE ROOT NAME OF THE STATPAC '',
     1''INPUT FILE? '',\)')
      READ(*,20) UNIT14
   20 FORMAT(A32)
      CALL CHEKER(UNIT14,EXT0,FLAG)
      IF(FLAG) GO TO 10
C -----------------------------------------------------------------------
C***  MAKE SURE INPUT FILE EXISTS BEFORE ATTEMPTING TO OPEN
C -----------------------------------------------------------------------
      CALL CKSAFE(UNIT14,2,FLAG)
      IF(FLAG) GO TO 10
      OPEN(KI,FILE=UNIT14,STATUS='OLD',FORM='UNFORMATTED')
   30 WRITE(*,'('' '',''WHAT IS THE ROOT NAME OF THE VARIOGRAM '',
     1''CONTROL FILE? '',\)')
      READ(*,20) UNIT20
      CALL CHEKER(UNIT20,EXT1,FLAG)
      IF(FLAG) GO TO 30
      CALL CKSAFE(UNIT20,2,FLAG)
      IF(FLAG) GO TO 30
      OPEN(IPT,FILE=UNIT20,STATUS='OLD',FORM='FORMATTED')
   40 WRITE(*,'('' '',''GIVE A ROOT NAME FOR THE VARIOGRAM OUTPUT '',
     1''FILE: '',\)')
      READ(*,20) UNIT21
      CALL CHEKER(UNIT21,EXT2,FLAG)
      IF(FLAG) GO TO 40
C -----------------------------------------------------------------------
C***  CHECK IF IT IS SAFE TO OPEN THE FILE FOR OUTPUT
C -----------------------------------------------------------------------
      CALL CKSAFE(UNIT21,1,FLAG)
      IF(FLAG) GO TO 40
      OPEN(IOUT,FILE=UNIT21,STATUS='NEW',FORM='FORMATTED')
      RETURN
      END
      SUBROUTINE RDN
C -----------------------------------------------------------------------
C***  THIS SUBROUTINE READS  DATA FROM A STATPAC INPUT FILE
C***  FOR PROGRAM SS2DGAMH.
C***  THIS PROGRAM REJECTS QUALIFIED VALUES.
C -----------------------------------------------------------------------
      CHARACTER*10 HMS
      CHARACTER*8 IVID(99),NAME(2),IDSTP,MDY
      CHARACTER*4 WKDAY,IROB
      CHARACTER*1 IA(99)
      INTEGER*4 LOC(2),NUMCUM
      COMMON XE(1500),YN(1500),GD(1500),IDATA(1500),AVGDIS(5,20),
     1DRIFT(5,20),NUMCUM(5,20),GAMMAH(5,20),CTRMNT(5,20),
     2HEAD(20),LOG,CLAS,DLIM,ANG(5),GDBAR,VARI,HMS,MDY,WKDAY,
     3STD,N,IPC,LL,SPR(5),ISCAL,TMAX,NREL,IOUT,IPT,IROB,
     4IPCH,KI,IDSTP(2),MAXN,NDIR,ISEL,XMIN,XMAX,YMIN,YMAX
      DIMENSION Z(99),IPACK(20)
      NHOL=0
      READ(KI) IDSTP(1),N,M,(IVID(I),I=1,M)
      NPACK=(M+4)/5
      WRITE(*,10) IDSTP(1)
   10 FORMAT(' ',//, 5X,'STATPAC INPUT FILE ID IS: ',A8,
     1' ;  COLUMN ID''S ARE:',/)
      WRITE(*,20) (I,IVID(I),I=1,M)
   20 FORMAT(5(1X,I2,')',A8))
   30 WRITE(*,'(/,5X,''ENTER STATPAC FILE DESCRIPTORS'',/)')
   40 WRITE(*,50)
   50 FORMAT(5X,'WHAT IS COLUMN NUMBER OF THE NORTH COORDINATE? : ',\)
      CALL ERRCHK(1,NO,DUMMY)
      IF(NO.GT.0.AND.NO.LE.M) GO TO 80
      CALL SOUND(256,100)
      WRITE(*,60)
   60 FORMAT(' ','SPECIFIED COLUMN DOES NOT EXIST; REENTER VALUE')
      GO TO 40
   70 FORMAT(BN,I2)
   80 WRITE(*,90)
   90 FORMAT(5X,'WHAT IS COLUMN NUMBER OF THE  EAST COORDINATE? : ',\)
      CALL ERRCHK(1,IE,DUMMY)
      IF(IE.GT.0.AND.IE.LE.M) GO TO 100
      CALL SOUND(256,100)
      WRITE(*,60)
      GO TO 80
  100 WRITE(*,110)
  110 FORMAT(5X,'WHAT IS COLUMN NUMBER OF THE ASSAY VALUE?      : ',\)
      CALL ERRCHK(1,LD,DUMMY)
      IF(LD.GT.0.AND.LD.LE.M) GO TO 120
      CALL SOUND(256,100)
      WRITE(*,60)
      GO TO 100
  120    IF(IE.EQ.NO.OR.IE.EQ.LD.OR.NO.EQ.LD) THEN
         CALL SOUND(256,100)
         WRITE(*,'('' '',''ERROR**  SPECIFIED COLUMNS NOT UNIQUE.  '',
     1   ''REDO FROM START'',/)')
         GO TO 40
         ENDIF
      IDSTP(2)=IVID(LD)
C -----------------------------------------------------------------------
C***  READ STATPAC INPUT FILE AND CHECK SAMPLES FOR ACCEPTABILITY.
C -----------------------------------------------------------------------
      WRITE(*,130)
  130 FORMAT(//,'        ....READING STATPAC INPUT FILE')
      CALL FNDCUR(IROW,ICOL,0)
         DO 150 I=1,N
         READ(KI) IRRR,NAME,LOC,(Z(K),K=1,M),(IPACK(K),K=1,NPACK)
         CALL UNPK(IA,M,IPACK)
         IF (ISEL.EQ.0)GO TO 140
         IF((Z(IE).LT.XMIN).OR.(Z(IE).GT.XMAX))GO TO 150
         IF((Z(NO).LT.YMIN).OR.(Z(NO).GT.YMAX))GO TO 150
C -----------------------------------------------------------------------
C***  REJECT QUALIFIED VALUES
C -----------------------------------------------------------------------
  140    IF(IA(LD).NE.' '.OR.IA(NO).NE.' '.OR.IA(IE).NE.' ')
     1   GO TO 150
         NHOL=NHOL+1
         IF(MOD(NHOL,10).EQ.0) CALL ODOMTR(IROW,NHOL)
         IF(NHOL.GT.MAXN)GO TO 190
         GD(NHOL)=Z(LD)
         YN(NHOL)=Z(NO)
         XE(NHOL)=Z(IE)
  150    CONTINUE
      WRITE(*,160) NHOL
  160 FORMAT('+',I5,' UNQUALIFIED DATA POINTS HAVE BEEN STORED')
      IREJ=N-NHOL
      IF(IREJ.NE.0) WRITE(*,170) IREJ
  170 FORMAT(' ',I5,' QUALIFIED DATA POINTS HAVE BEEN REJECTED')
      IF(NHOL.GT.1) GO TO 210
         DO 180 I=1,5
         CALL SOUND(256,100)
  180    CONTINUE
  190 WRITE(IOUT,200) NHOL,MAXN
  200 FORMAT(' INPUT DATA ERROR',3X,I5,' DATA POINTS STORED',/,
     1' THE LIMIT IS ',I5)
      STOP
  210 N=NHOL
      RETURN
      END
      SUBROUTINE VAROUT(GAMVAL)
C -----------------------------------------------------------------------
C***  THIS SUBROUTINE PRINTS THE RESULTS OF PROGRAM SS2DGAMH.
C -----------------------------------------------------------------------
      CHARACTER*10 HMS
      CHARACTER*8 MDY,IDSTP
      CHARACTER*4 HEAD,ILOG,WKDAY,IROB
      CHARACTER*1 D,E,BLANK,A
      INTEGER*4 NUMCUM,LOW,LUP
      LOGICAL IAMP
      DIMENSION A(50),C(6),D(20),E(13),ILOG(3),GAMVAL(5,20)
      COMMON XE(1500),YN(1500),GD(1500),IDATA(1500),AVGDIS(5,20),
     1DRIFT(5,20),NUMCUM(5,20),GAMMAH(5,20),CTRMNT(5,20),
     2HEAD(20),LOG,CLAS,DLIM,ANG(5),GDBAR,VARI,HMS,MDY,WKDAY,
     3STD,N,IPC,LL,SPR(5),ISCAL,TMAX,NREL,IOUT,IPT,IROB,
     4IPCH,KI,IDSTP(2),MAXN,NDIR,ISEL,XMIN,XMAX,YMIN,YMAX
      DATA ILOG/'NO','YES','YES'/, BLANK/' '/
      DATA D/5*' ','G','A','M','M','A',' ',' ','*','H','*',5*' '/
      DATA E/'M','O','M','E','N','T',' ','C','E','N','T','E','R'/
         DO 10 I=1,50
   10    A(I)=BLANK
      IF(LL.EQ.0)GO TO 30
         DO 20 I=1,13
         D(I+5)=E(I)
   20    CONTINUE
   30 C(1) = 0.
         DO 40 I=2,6
         C(I)=CLAS*4.+C(I-1)
   40    CONTINUE
C -----------------------------------------------------------------------
C***  WRITE TABULAR RESULTS ON DISK FILE
C -----------------------------------------------------------------------
      IF(IPC.NE.0) WRITE(IPCH,'(2A8,/,I8,1X,''20'')') IDSTP,NDIR
         DO 200 L=1,NDIR
         IAMP=.FALSE.
         WRITE(IOUT,'(//,''VARIOGRAM'')')
         WRITE(IOUT,50) WKDAY,MDY,HMS,IDSTP(2)
   50    FORMAT(6X,A4,2X,A8,2X,A10,5X,'VARIABLE ID: ',A8)
         NL=LOG+1
         NR=NREL+1
         WRITE(IOUT,60)HEAD
   60    FORMAT(' ',19A4,A3)
         WRITE(IOUT,'(54X,''DATA USED IN CALCULATIONS'')')
         WRITE(IOUT,70) ANG(L),SPR(L),GDBAR,CLAS,VARI,DLIM,STD,
     1   ILOG(NL),ILOG(NR),N,IROB
   70    FORMAT(' DIRECTION    =',F5.0,3X,'WINDOW',F5.0,20X,'MEAN',
     1   10X,'=',E10.3,/' CLASS SIZE  =',F7.1,33X,'VARIANCE      =',
     2   E10.3,/' MAX. DISTANCE = ',F10.0,27X,'STD DEVIATION =',E10.3,
     3   / ' LOGARITHMS -',A4,5X,'RELATIVE VARIOGRAM -',A4,8X,
     4   'NO.OF SAMPLES =',I7,/,' ROBUST -',A4)
         IF(ISEL.NE.0)WRITE(IOUT,80)YMIN,YMAX,XMIN,XMAX
   80    FORMAT(' COORDINATE SELECTION- NORTH(',2F8.0,')'
     1   3X,'EAST(',2F8.0,')')
         WRITE(IOUT,90)
   90    FORMAT(' ',//,4X,'DISTANCE',3X,'  PAIRS',4X,'"DRIFT"',
     1   4X,' GAMMA (H)',1X,'MOMENT CENT',1X,'AVER DIST')
            DO 120 I=1,20
            LOW=IFIX(FLOAT(I-1)*CLAS)
            LUP=IFIX(FLOAT(I)*CLAS)
            IF(NUMCUM(L,I).GT.0)WRITE(IOUT,100)LOW,LUP,NUMCUM(L,I),
     1      DRIFT(L,I),GAMMAH(L,I),CTRMNT(L,I),AVGDIS(L,I)
  100       FORMAT(1H ,I6,' -',I6,I8,3(2X,E10.3),F10.2)
C -----------------------------------------------------------------------
C***  AS AN OPTION, CREATE DISK FILE OF VARIOGRAM DATA FOR PLOTTER.
C -----------------------------------------------------------------------
            IF(IPC.EQ.1) WRITE(IPCH,110) AVGDIS(L,I),GAMVAL(L,I)
  110       FORMAT(E10.3,1X,E10.3)
  120       CONTINUE
C -----------------------------------------------------------------------
C***  WRITE GRAPHIC RESULTS ON DISK FILE.
C -----------------------------------------------------------------------
         WRITE(IOUT,130)
         WRITE(IOUT,50) WKDAY,MDY,HMS,IDSTP(2)
         WRITE(IOUT,60)HEAD
         WRITE(IOUT,130)
  130    FORMAT(//)
            IF(ISCAL.NE.0.AND.TMAX.EQ.0.) THEN
C -----------------------------------------------------------------------
C***  AUTOMATIC SCALING IS DESIRED
C -----------------------------------------------------------------------
            TMAX=GAMVAL(L,1)
               DO 140 I=2,20
               IF(GAMVAL(L,I).GT.TMAX)TMAX=GAMVAL(L,I)
  140          CONTINUE
            ENDIF
         UNIT=CLAS/2.5
         DIV=TMAX/20.
         TEMP=TMAX
            DO 180 K=1,20
            TOP=TEMP
            BOT=TEMP-DIV
               DO 150 I=1,20
               IF(NUMCUM(L,I).EQ.0)GO TO 150
               IF((GAMVAL(L,I).GT.TOP).OR.(GAMVAL(L,I).LE.BOT))GO TO 150
               J=AVGDIS(L,I)/UNIT +1
               A(J)='X'
                  IF(NUMCUM(L,I).LT.30) THEN
                  A(J)='&'
                  IAMP=.TRUE.
                  ENDIF
  150          CONTINUE
C -----------------------------------------------------------------------
C***  ARRAY CONTAINS BLANKS AS WELL AS DATA POINT MARKERS
C -----------------------------------------------------------------------
            WRITE(IOUT,160)D(K),TEMP,A
  160       FORMAT(1H ,1X,A1,1X,E10.3,' *',50A1)
            TEMP=BOT
               DO 170 I=1,50
               A(I)=BLANK
  170          CONTINUE
  180       CONTINUE
         WRITE(IOUT,190)
  190    FORMAT(1H ,15X,5('+----+----'),'+')
         WRITE(IOUT,'('' '',10X,7(F7.0,3X))') C
         WRITE(IOUT,'('' '',36X,''DISTANCE'')')
         IF(IAMP) WRITE(IOUT,'(/,21X,''AMPERSANDS (&) DENOTE FEWER '',
     1   ''THAN 30 PAIRS'')')
  200    CONTINUE
      RETURN
      END

